# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : PC_elements_MTY.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_Control
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_ORD, _COL_MTY

list_Controls_DSS = ['CapControl', 'ESPVLControl', 'ExpControl', 'Fuse', 'GenDispatcher', 'InvControl', 'Recloser',
                     'RegControl', 'Relay', 'StorageController', 'SwtControl', 'UPFCControl']

def Controls_ORD(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:
    if name_class == 'CapControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'ESPVLControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'ExpControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'Fuse':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'GenDispatcher':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'InvControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'Recloser':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'RegControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'Relay':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'StorageController':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'SwtControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    elif name_class == 'UPFCControl':
        return _COL_ORD(dict_Control, DF_elem_DSS, name_class)
    else:
        return DF_elem_DSS


def Controls_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:
    if name_class == 'CapControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'ESPVLControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'ExpControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Fuse':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'GenDispatcher':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'InvControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Recloser':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'RegControl':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                row_aux = [
                    'revvreg', 'band', 'ptratio', 'CTprim', 'delay', 'reversible', 'tapdelay', 'debugtrace',
                    'maxtapchange', 'inversetime', 'vlimit', 'PTphase', 'revThreshold', 'revDelay', 'revNeutral',
                    'EventLog', 'TapNum', 'Cogen', 'basefreq', 'enabled', 'revband']

                value_aux = [
                    120, 3, 60, 0.2, 15, 'No', 2, 'No', 16, 'No', 0, 1, 100, 60, 'No', 'Yes', 0, 'No', 'basefreq',
                    'Yes', 3]

                for x in zip(row_aux, value_aux):

                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''

        return DF_elem_DSS

    elif name_class == 'Relay':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'StorageController':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'SwtControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'UPFCControl':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    else:
        return DF_elem_DSS

def Controls_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):
    # list_property = dss.dsselement_all_property_names()
    list_property = dict_Control[name_class]

    if name_class == 'CapControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'ESPVLControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'ExpControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Fuse':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'GenDispatcher':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'InvControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Recloser':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'RegControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Relay':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'StorageController':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'SwtControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'UPFCControl':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return BBDD_elem_DSS, DSS_elem_list





# -*- coding: utf-8 -*-
# @Time    : 26/08/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : load_examples.py
# @Software: PyCharm

import logging
import os
import pathlib
from openpy_fx_tools_dss.logg_print_alert import logg_alert
log_py = logging.getLogger(__name__)


class examples_lib:

    def load_examples_DSS(self, Option: int = None, case: str = None) -> dict:
        """
        Within the module, look for the selected option in the examples tab. Return a dictionary with the keys
        ['DSS_path'], ['path_save'] and ['prj_name'].

        :param Option: Option select. Default is None.
        :param case:

        :return: dict_DSS
        """
        dict_DSS = _test_DSS_files(Opt=Option, case=case)

        return dict_DSS

    def load_examples_xlsx(self, Option: int = None) -> dict:
        """
        Within the module, look for the selected option in the examples tab. Return a dictionary with the keys
        ['xlsx_path'], ['path_save'] and ['prj_name'].

        :param Option: Option select. Default is None.
        :return: dict_xlsx
        """
        dict_xlsx = _test_xlsx_files(Opt=Option)

        return dict_xlsx

def _test_DSS_files(Opt: int, case: str):
    script_path = os.path.dirname(os.path.abspath(__file__))
    DSS_info = dict()
    if case == None:
        case = "."
    else:
        case = case + '_files'

    if Opt == 1:
        DSS_info['DSS_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "13Bus", "DSS_files", "IEEE13Nodeckt.dss")
        DSS_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "13Bus", case)
        DSS_info['prj_name'] = '13BusIEEE'

    if Opt == 2:
        DSS_info['DSS_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "37Bus", "DSS_files", "ieee37.dss")
        DSS_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "37Bus", case)
        DSS_info['prj_name'] = '37BusIEEE'

    if Opt == 3:
        DSS_info['DSS_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "123Bus", "DSS_files", "IEEE123Master")
        DSS_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "123Bus", case)
        DSS_info['prj_name'] = '123BusIEEE'

    res = not DSS_info
    if res:
        logg_alert.update_logg_file('Select an uploaded example, see documentation', 4, log_py)
        exit()

    return DSS_info


def _test_xlsx_files(Opt: int) -> object:
    script_path = os.path.dirname(os.path.abspath(__file__))
    xlsx_info = dict()
    if Opt == 1:
        xlsx_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "13Bus", "xlsx_files", "DSS_files_from_xlsx")
        xlsx_info['xlsx_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "13Bus", "xlsx_files", "BBDD_DSS_13BusIEEE.xlsx")

    if Opt == 2:
        xlsx_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "37Bus", "xlsx_files", "DSS_files_from_xlsx")
        xlsx_info['xlsx_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "37Bus", "xlsx_files", "BBDD_DSS_37BusIEEE.xlsx")

    if Opt == 3:
        xlsx_info['path_save'] = pathlib.Path(script_path).joinpath(
            "./Examples", "123Bus", "xlsx_files", "DSS_files_from_xlsx")
        xlsx_info['xlsx_path'] = pathlib.Path(script_path).joinpath(
            "./Examples", "123Bus", "xlsx_files", "BBDD_DSS_123BusIEEE.xlsx")

    res = not xlsx_info
    if res:
        logg_alert.update_logg_file('Select an uploaded example, see documentation', 4, log_py)
        exit()

    return xlsx_info





# -*- coding: utf-8 -*-
# @Time    : 26/10/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import logging
import pandas as pd
from io import open
import numpy as np
import openpyxl
import os
import glob
import shutil
from openpy_fx_tools_dss.logg_print_alert import logg_alert
from ...helper_functions import is_float

log_py = logging.getLogger(__name__)



def create_scrips_base_dss(name_dss: str, xlsx_file: str, path_save: str, add_path: bool, coords: str):
    """
    function that creates the OpenDSS scripts with the information from the database.

    :param name_dss:
    :param xlsx_file:
    :param path_save:
    :param add_path:
    :param coords:
    :return: OpenDSS scripts in the specified path_save
    """
    delete_all_files(path_save)
    create_master_base_dss(name_dss, xlsx_file, path_save, add_path, coords)
    for element in check_BBDD(xlsx_file):
        create_element_base_dss(name_dss, element, xlsx_file, coords)
    logg_alert.update_logg_file('_' * 64, 1)
    logg_alert.update_logg_file('Data loaded and .DSS files created', 2, log_py)


def create_cases(name_dss: str, file_BBDD: str, root_path: str, name_folder: str):
    """

    :param name_dss:
    :param file_BBDD:
    :param root_path:
    :param name_folder:
    :return:
    """
    indicator = os.path.exists(name_folder)

    if indicator:
        shutil.rmtree(name_folder)
        create_folder(name_folder)
        delete_files_DSS()
        root_file_path = root_path+f'/{file_BBDD}'
        root_file_path_csv = root_path + f'/Buscoords_{name_dss}.csv'
        name_dss_aux = name_dss + f'_{name_folder}'
        create_scrips_dss(name_dss_aux, file_BBDD, root_file_path)
        move_files_DSS(root_path, root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path, dst=root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path_csv, dst=root_path + f'/{name_folder}')
        os.rename(root_path + f'/{name_folder}' + f'/{file_BBDD}', root_path+ f'/{name_folder}' + f'/BBDD_{name_folder}.xlsx')
        os.rename(root_path + f'/{name_folder}' + f'/Buscoords_{name_dss}.csv', root_path + f'/{name_folder}' + f'/Buscoords_{name_dss}_{name_folder}.csv')

    elif not indicator:
        create_folder(name_folder)
        delete_files_DSS()
        root_file_path = root_path + f'/{file_BBDD}'
        root_file_path_csv = root_path + f'/Buscoords_{name_dss}.csv'
        name_dss_aux = name_dss + f'_{name_folder}'
        create_scrips_dss(name_dss_aux, file_BBDD, root_file_path)
        move_files_DSS(root_path, root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path, dst=root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path_csv, dst=root_path + f'/{name_folder}')
        os.rename(root_path + f'/{name_folder}' + f'/{file_BBDD}', root_path + f'/{name_folder}' + f'/BBDD_{name_folder}.xlsx')
        os.rename(root_path + f'/{name_folder}' + f'/Buscoords_{name_dss}.csv', root_path + f'/{name_folder}' + f'/Buscoords_{name_dss}_{name_folder}.csv')

    else:
        print('Cannot perform the operation try another name')

def create_folder(name_folder: str):
    os.mkdir(name_folder)

def file_path(workbook):
    return read_direction()+workbook

def read_direction():
    file_path = os.path.dirname(os.path.abspath(__file__))
    file_path = file_path.replace("\\", "/")
    return file_path

def move_files_DSS(source_address, destination_address):
    """
    :return:
    Delete existing .DSS files in the specified path_save
    """
    direction = read_direction()
    dss_files = glob.glob(direction+'/*.dss')
    dss_files2 = glob.glob('/*.dss')
    for dss_file in dss_files:
        shutil.move(dss_file, destination_address)

#1
def delete_all_files(path_save):
    """
    Delete existing .DSS files in the specified path_save
    
    :param path_save:
    :return: 
    """
    dss_files = glob.glob(f'{path_save}/*.dss')
    for dss_file in dss_files:
        try:
            os.remove(dss_file)
        except OSError as e:
            print(f"Error:{e.strerror}")

def delete_files_DSS():
    """
    :return:
    Delete existing .DSS files in the specified path_save
    """
    path_save = read_direction()
    dss_files = glob.glob(f'{path_save}/*.dss')
    for dss_file in dss_files:
        try:
            os.remove(dss_file)
        except OSError as e:
            print(f"Error:{e.strerror}")

def master_content_dir(name_dss: str, list_elements: list, path_save: str, add_path: bool, coords: str):
    """

    :param name_dss:
    :param list_elements:
    :param path_save:
    :param add_path:
    :param coords:
    :return:
    """
    if add_path:
        content = \
            f'set Datapath=({path_save}\)\n'\
            'Clear\n'\
            '\n'\
            f'New Circuit.{name_dss}\n'\
            '\n'
    else:
        content = \
            'Clear\n' \
            '\n' \
            f'New Circuit.{name_dss}\n' \
            '\n'
    aux = ''
    content_aux = ''

    for element in list_elements:
        if element == 'Buscoords':
            pass
        elif element != 'Voltagebases':
            content_aux = f'Redirect {element}_{name_dss}.dss\n'
            aux = aux + content_aux

    aux = content + aux

    aux_0 = '\n'\
            f'Redirect Voltagebases_{name_dss}.dss\n'
    if logg_alert.export_csv:
        if coords == 'XY':
            aux_1 = '\n'\
                    f'Buscoords Buscoords_{name_dss}.csv' \
                    '\n'
        elif coords == 'LatLong':
            aux_1 = '\n'\
                    f'LatLongCoords Buscoords_{name_dss}.csv'\
                    '\n'
        else:
            aux_1 = '\n'
    else:
        if coords == 'XY':
            aux_1 = '\n'\
                    f'Buscoords Buscoords_{name_dss}.dss' \
                    '\n'
        elif coords == 'LatLong':
            aux_1 = '\n'\
                    f'LatLongCoords Buscoords_{name_dss}.dss'\
                    '\n'
        else:
            aux_1 = '\n'

    aux_2 = '\n'\
            'solve'

    aux = aux + aux_0 + aux_1 + aux_2

    return aux


#2
def create_master_base_dss(name_dss: str, xlsx_file: str, path_save: str, add_path: bool, coords: str):
    """

    :param name_dss:
    :param xlsx_file:
    :param path_save:
    :param add_path:
    :param coords:
    :return:
    """
    os.chdir(path_save)
    os.getcwd()
    list_elements = check_BBDD(xlsx_file)
    master_dss = open(f'Master_{name_dss}.dss', 'w')
    content = master_content_dir(name_dss, list_elements, path_save, add_path, coords)
    master_dss.write(content)
    master_dss.close()

def create_master_dss(name_dss: str, xlsx_file: str):

    list_elements = check_BBDD(xlsx_file)
    master_dss = open(f'Master_{name_dss}.dss', 'w')
    content = master_content(name_dss, list_elements)
    master_dss.write(content)
    master_dss.close()

#4
def check_BBDD(xlsx_file: str):
    """
    :param xlsx_file:
    :return:
    """
    workbook = openpyxl.load_workbook(xlsx_file, read_only=True)
    name_sheets = workbook.sheetnames
    list_elements = []
    for name_sheet in name_sheets:
        df_sheet = pd.read_excel(xlsx_file, sheet_name=name_sheet)
        if len(df_sheet) >= 1:
            list_elements.append(name_sheet)
        else:
            pass
    return list_elements

#5
def create_element_base_dss(name_dss: str, element: str, direction: str, coords: str):
    """
    :param name_dss:
    :param element:
    :param direction:
    :return:
    """
    df_element = pd.read_excel(direction, sheet_name='{}'.format(element))
    dict_element = df_element.to_dict()
    keys = [*dict_element]
    script = ''
    aux = 0
    list_aux = []

    if element != 'Buscoords':
        for n in range(len(dict_element['Id_{}'.format(element)])):
            for key in keys:
                if key == 'Id_{}'.format(element):
                    if dict_element[key][aux] == 'source':
                        script = 'Edit "{}.{}" '.format(element, dict_element[key][aux])
                    else:
                        if element == 'Switch':
                            element_aux = 'Line'
                            script = 'New "{}.{}" '.format(element_aux, dict_element[key][aux])
                        elif element == 'Voltagebases':
                            script = f'Set Voltagebases = {dict_element[key][aux]}\n calcv\n'
                        else:
                            script = 'New "{}.{}" '.format(element, dict_element[key][aux])

                    list_aux.append(script)
                else:
                    if aux <= len(dict_element[key]):
                        value_DSS = dict_element[key][aux]

                        if str(value_DSS) != 'nan':
                            list_str = ['linecode']
                            if len([x for x in [key] if x in list_str]) == 1:
                                if is_float(dict_element[key][aux]):
                                    value_DSS = str(int(value_DSS))

                            script = '{}={} '.format(key, value_DSS)
                            list_aux.append(script)
                        else:
                            pass
            list_aux.append('\n')

            aux += 1
        element_content = '{}'.format(''.join(list_aux))
        element_dss = open('{}_{}.dss'.format(element, name_dss), 'w')
        element_dss.write(element_content)
        element_dss.close()
    else:
        if coords == 'XY':
            df_element = df_element[['Bus name', 'X', 'Y']]
        elif coords == 'LatLong':
            df_element = df_element[['Bus name', 'Long', 'Lat']]
        else:
            df_element = df_element[['Bus name']]
            df_element['Aux_Y'] = 0
            df_element['Aux_X'] = 0
        element_content = df_element.to_csv(index=False, header=False).replace('\r', '')
        element_dss = open('{}_{}.dss'.format(element, name_dss), 'w')
        element_dss.write(element_content)
        element_dss.close()

def create_element_dss(name_dss: str, element: str, direction: str):
    """
    :param name_dss:
    :param element:
    :param direction:
    :return:
    """
    df_element = pd.read_excel(direction, sheet_name='{}'.format(element))
    dict_element = df_element.to_dict()
    keys = [*dict_element]
    script = ''
    aux = 0
    list_aux = []
    for n in range(len(dict_element['Id_{}'.format(element)])):
        for key in keys:
            if key == 'Id_{}'.format(element):
                if dict_element[key][aux] == 'source':
                    script = 'Edit "{}.{}" '.format(element, dict_element[key][aux])
                else:
                    if element == 'Switch':
                        element_aux = 'Line'
                        script = 'New "{}.{}" '.format(element_aux, dict_element[key][aux])
                    elif element == 'Voltagebases':
                        script = \
                            f'Set Voltagebases = {dict_element[key][aux]}\n'\
                            f'calcv\n'
                    else:
                        script = 'New "{}.{}" '.format(element, dict_element[key][aux])

                list_aux.append(script)
            else:
                if aux <= len(dict_element[key]):
                    if str(dict_element[key][aux]) != 'nan':
                        script = '{}={} '.format(key, dict_element[key][aux])
                        list_aux.append(script)
                    else:
                        pass
        list_aux.append('\n')

        aux += 1
    element_content = '{}'.format(''.join(list_aux))
    element_dss = open('{}_{}.dss'.format(element, name_dss), 'w')
    element_dss.write(element_content)
    element_dss.close()

def create_scrips_dss(name_dss: str, workbook: str, ruta_archivo: str):
    """
    :param name_dss:
    :param workbook:
    :param ruta_archivo:
    :return OpenDSS scripts in the specified path_save

    function that creates the OpenDSS scripts with the information from the database
    """
    delete_files_DSS()
    create_master_dss(name_dss, workbook)
    for element in check_BBDD(workbook):
        create_element_dss(name_dss, element, ruta_archivo)
    print('Data loaded and .DSS files created')

def create_scripts_OpenDSS(name_dss, workbook, ruta_raiz, name_folder):
    create_cases(name_dss, workbook, ruta_raiz, name_folder)


# -*- coding: utf-8 -*-
# @Time    : 26/10/2022 3:41 PM
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : interface_dss.py
# @Software: PyCharm

import py_dss_interface
import opendssdirect as drt


dss = py_dss_interface.DSSDLL()



# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : PC_elements_MTY.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_Meters
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_ORD, _COL_MTY

list_Meters_DSS = ['EnergyMeter', 'FMonitor', 'Monitor', 'Sensor']

def Meters_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:

    if name_class == 'EnergyMeter':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'FMonitor':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Monitor':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Sensor':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    else:
        return DF_elem_DSS

def Meters_ORD(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:

    if name_class == 'EnergyMeter':
        return _COL_ORD(dict_Meters, DF_elem_DSS, name_class)
    elif name_class == 'FMonitor':
        return _COL_ORD(dict_Meters, DF_elem_DSS, name_class)
    elif name_class == 'Monitor':
        return _COL_ORD(dict_Meters, DF_elem_DSS, name_class)
    elif name_class == 'Sensor':
        return _COL_ORD(dict_Meters, DF_elem_DSS, name_class)
    else:
        return DF_elem_DSS

def Meters_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):
    #list_property = dss.dsselement_all_property_names()
    list_property = dict_Meters[name_class]

    if name_class == 'EnergyMeter':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'FMonitor':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Monitor':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Sensor':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return BBDD_elem_DSS, DSS_elem_list





# -*- coding: utf-8 -*-
# @Time    : 27/06/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss

def from_to_bus(type_element, num_cond, node_order, bus1, bus2):
    if type_element == True:
        if num_cond == 1:
            aux1 = bus1.find('.' + str(node_order[0]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]
            to_bus = ''
        elif num_cond == 2:
            aux1 = bus1.find('.' + str(node_order[0]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]
            to_bus = ''
        elif num_cond == 3:
            aux1 = bus1.find('.' + str(node_order[0]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]
            to_bus = ''
        elif num_cond == 4:
            aux1 = bus1.find('.' + str(node_order[0]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]
            to_bus = ''

    else:
        if num_cond == 1:
            aux1 = bus1.find('.' + str(node_order[0]))
            aux2 = bus2.find('.' + str(node_order[1]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]

            if aux2 == -1:
                to_bus = bus2
            else:
                to_bus = bus2[:aux2]
        elif num_cond == 2:
            aux1 = bus1.find('.' + str(node_order[0]))
            aux2 = bus2.find('.' + str(node_order[2]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]

            if aux2 == -1:
                to_bus = bus2
            else:
                to_bus = bus2[:aux2]

        elif num_cond == 3:
            aux1 = bus1.find('.' + str(node_order[0]))
            aux2 = bus2.find('.' + str(node_order[3]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]

            if aux2 == -1:
                to_bus = bus2
            else:
                to_bus = bus2[:aux2]
        elif num_cond == 4:
            aux1 = bus1.find('.' + str(node_order[0]))
            aux2 = bus2.find('.' + str(node_order[4]))
            if aux1 == -1:
                from_bus = bus1
            else:
                from_bus = bus1[:aux1]
            if aux2 == -1:
                to_bus = bus2
            else:
                to_bus = bus2[:aux2]

    return from_bus, to_bus

def characters_delete(DataFrame: pd.DataFrame, column: str) -> object:
    """
    Function that eliminates empty spaces and special characters that can cause convergence problems in OpenDSS

    :param DataFrame: Name of the selected dataFrame
    :param column: Column name of the selected dataFrame
    :return:
    """
    characters_delete_space = ['.1', '.2', '.3', '.4']
    for i in characters_delete_space:
        DataFrame[column] = DataFrame[column].replace({i: ''}, regex=True)

def save_BBDD_xlsx(workbook_DSS: str, elements_OpenDSS: list, BBDD_OpenDSS: dict, out_path:str):
    """
    Generates the .xlsx file, with xlsx_data format for OpenDSS

    :param workbook_DSS:
    :param elements_OpenDSS:
    :param BBDD_OpenDSS:
    :param out_path:
    :return:
    """

    writer = pd.ExcelWriter(f'{out_path}\{workbook_DSS}')
    for name in elements_OpenDSS:
        BBDD_OpenDSS[name].to_excel(writer, sheet_name=name, index=False)
    writer.save()
    writer.close()

def Update_coordinates_for_GIS(prj_name: str, DSS_file: str, save_path: str):
    '''
    Function that generates a .xlsx file with the coordinates for GIS of a circuit modeled in OpenDSS

    :param prj_name: Name of the new .xlsx file
    :param DSS_file: path_save where the OpenDSS file is located
    :param save_path: path_save where it is going to save
    :return:
    '''

    element_list = list()
    BBDD_Buscoords = dict()

    dss.text("ClearAll")
    dss.text(f"compile [{DSS_file}]")
    dss.solution_solve()

    df_Buscoords = pd.DataFrame(columns=['Id_node', 'GPSlat(r)', 'GPSlon(r)'])
    df_Vsource = pd.DataFrame(columns=['Id_Vsource', 'bus1'])
    df_Transformer = pd.DataFrame(columns=['Id_Transformer', 'bushv', 'buslv'])
    df_Load = pd.DataFrame(columns=['Id_Load', 'bus1'])
    df_Capacitor = pd.DataFrame(columns=['Id_Capacitor', 'bus1'])
    df_Line = pd.DataFrame(columns=['Id_Line', 'bus1', 'bus2'])
    df_Switch = pd.DataFrame(columns=['Id_Switch', 'bus1', 'bus2'])
    df_PVSystem = pd.DataFrame(columns=['Id_PVSystem', 'bus1'])

    list_bus_names = dss.circuit_all_bus_names()
    for bus in list_bus_names:
        dss.circuit_set_active_bus(bus)
        longitude = dss.bus_read_y()
        latitud = dss.bus_read_x()
        df_Buscoords = df_Buscoords.append(
            {'Id_node': bus, 'GPSlat(r)': latitud, 'GPSlon(r)': longitude}, ignore_index=True)
    elements = ['vsources', 'transformers', 'lines', 'loads', 'capacitors']
    BBDD_Buscoords['Buscoords'] = df_Buscoords
    element_list.append('Buscoords')

    'Vsource'
    num_element = dss.vsources_count()
    dss.vsources_first()
    for num in range(num_element):
        name = dss.vsources_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = \
            dss.cktelement_num_conductors(), \
            dss.cktelement_node_order(), \
            dss.cktelement_read_bus_names()[0], ''
        from_bus, to_bus = from_to_bus(False, num_cond, node_order, bus1, bus2)

        df_Vsource = df_Vsource.append({'Id_Vsource': name, 'bus1': from_bus}, ignore_index=True)

    df_Vsource = pd.merge(df_Vsource, df_Buscoords, how='left', left_on='bus1', right_on='Id_node').rename(
        columns={'GPSlat(r)': 'GPSlat(r)_bus1', 'GPSlon(r)': 'GPSlon(r)_bus1'})
    df_Vsource = df_Vsource[['Id_Vsource', 'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1']]

    BBDD_Buscoords['Vsource'] = df_Vsource
    element_list.append('Vsource')

    'transformers'
    num_element = dss.transformers_count()
    dss.transformers_first()
    for num in range(num_element):
        name = dss.vsources_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = dss.cktelement_num_conductors(), dss.cktelement_node_order(), \
                                           dss.cktelement_read_bus_names()[0], dss.cktelement_read_bus_names()[1]
        from_bus, to_bus = from_to_bus(False, num_cond, node_order, bus1, bus2)

        df_Transformer = df_Transformer.append(
            {'Id_Transformer': name, 'bushv': from_bus, 'buslv': to_bus}, ignore_index=True)

        dss.transformers_next()

    df_Transformer = pd.merge(df_Transformer, df_Buscoords, how='left', left_on='bushv', right_on='Id_node')
    df_Transformer = pd.merge(df_Transformer, df_Buscoords, how='left', left_on='buslv', right_on='Id_node').rename(
        columns={'GPSlat(r)_x': 'GPSlat(r)_bushv', 'GPSlon(r)_x': 'GPSlon(r)_bushv',
                 'GPSlat(r)_y': 'GPSlat(r)_buslv', 'GPSlon(r)_y': 'GPSlon(r)_buslv'})

    df_Transformer = df_Transformer[
        ['Id_Transformer', 'bushv', 'GPSlat(r)_bushv', 'GPSlon(r)_bushv', 'buslv', 'GPSlat(r)_buslv', 'GPSlon(r)_buslv']
    ]

    BBDD_Buscoords['Transformer'] = df_Transformer
    element_list.append('Transformer')

    'Load'
    num_element = dss.loads_count()
    dss.loads_first()
    for num in range(num_element):
        name = dss.loads_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = dss.cktelement_num_conductors(), dss.cktelement_node_order(), \
                                           dss.cktelement_read_bus_names()[0], ''
        from_bus, to_bus = from_to_bus(True, num_cond, node_order, bus1, bus2)
        df_Load = df_Load.append({'Id_Load': name, 'bus1': from_bus}, ignore_index=True)
        dss.loads_next()

    df_Load = pd.merge(df_Load, df_Buscoords, how='left', left_on='bus1', right_on='Id_node').rename(
        columns={'GPSlat(r)': 'GPSlat(r)_bus1', 'GPSlon(r)': 'GPSlon(r)_bus1'})
    df_Load = df_Load[['Id_Load', 'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1']]

    BBDD_Buscoords['Load'] = df_Load
    element_list.append('Load')

    'Capacitor'
    num_element = dss.capacitors_count()
    dss.capacitors_first()
    for num in range(num_element):
        name = dss.capacitors_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = dss.cktelement_num_conductors(), dss.cktelement_node_order(), \
                                           dss.cktelement_read_bus_names()[0], ''
        from_bus, to_bus = from_to_bus(False, num_cond, node_order, bus1, bus2)
        df_Capacitor = df_Capacitor.append({'Id_Capacitor': name, 'bus1': from_bus}, ignore_index=True)
        dss.capacitors_next()

    df_Capacitor = pd.merge(df_Capacitor, df_Buscoords, how='left', left_on='bus1', right_on='Id_node').rename(
        columns={'GPSlat(r)': 'GPSlat(r)_bus1', 'GPSlon(r)': 'GPSlon(r)_bus1'})
    df_Capacitor = df_Capacitor[['Id_Capacitor', 'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1']]

    BBDD_Buscoords['Capacitor'] = df_Capacitor
    element_list.append('Capacitor')

    'Line'
    num_element = dss.lines_count()
    dss.lines_first()
    for num in range(num_element):
        name = dss.lines_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = dss.cktelement_num_conductors(), dss.cktelement_node_order(), \
                                           dss.cktelement_read_bus_names()[0], dss.cktelement_read_bus_names()[1]
        from_bus, to_bus = from_to_bus(False, num_cond, node_order, bus1, bus2)

        switch = dss.dssproperties_read_value('15')
        if switch == 'False':
            df_Line = df_Line.append(
                {'Id_Line': name, 'bus1': from_bus, 'bus2': to_bus}, ignore_index=True)

        if switch == 'True':
            df_Switch = df_Switch.append(
                {'Id_Switch': name, 'bus1': from_bus, 'bus2': to_bus}, ignore_index=True)

        dss.lines_next()
    df_Line = pd.merge(df_Line, df_Buscoords, how='left', left_on='bus1', right_on='Id_node')
    df_Line = pd.merge(df_Line, df_Buscoords, how='left', left_on='bus2', right_on='Id_node').rename(
        columns={'GPSlat(r)_x': 'GPSlat(r)_bus1', 'GPSlon(r)_x': 'GPSlon(r)_bus1',
                 'GPSlat(r)_y': 'GPSlat(r)_bus2', 'GPSlon(r)_y': 'GPSlon(r)_bus2'})

    df_Line = df_Line[['Id_Line',
                       'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1',
                       'bus2', 'GPSlat(r)_bus2', 'GPSlon(r)_bus2']]

    BBDD_Buscoords['Line'] = df_Line
    element_list.append('Line')

    df_Switch = pd.merge(df_Switch, df_Buscoords, how='left', left_on='bus1', right_on='Id_node')
    df_Switch = pd.merge(df_Switch, df_Buscoords, how='left', left_on='bus2', right_on='Id_node').rename(
        columns={'GPSlat(r)_x': 'GPSlat(r)_bus1', 'GPSlon(r)_x': 'GPSlon(r)_bus1',
                 'GPSlat(r)_y': 'GPSlat(r)_bus2', 'GPSlon(r)_y': 'GPSlon(r)_bus2'})

    df_Switch = df_Switch[['Id_Switch',
                           'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1',
                           'bus2', 'GPSlat(r)_bus2', 'GPSlon(r)_bus2']]
    BBDD_Buscoords['Switch'] = df_Switch
    element_list.append('Switch')

    'PVSystem'
    num_element = dss.pvsystems_count()
    dss.pvsystems_first()
    for num in range(num_element):
        name = dss.pvsystems_read_name()
        bus_name = dss.cktelement_read_bus_names()
        num_cond, node_order, bus1, bus2 = dss.cktelement_num_conductors(), dss.cktelement_node_order(), \
                                           dss.cktelement_read_bus_names()[0], ''
        from_bus, to_bus = from_to_bus(False, num_cond, node_order, bus1, bus2)
        df_PVSystem = df_PVSystem.append({'Id_PVSystem': name, 'bus1': from_bus}, ignore_index=True)
        dss.pvsystems_next()

    df_PVSystem = pd.merge(df_PVSystem, df_Buscoords, how='left', left_on='bus1', right_on='Id_node').rename(
        columns={'GPSlat(r)': 'GPSlat(r)_bus1', 'GPSlon(r)': 'GPSlon(r)_bus1'})
    df_PVSystem = df_PVSystem[['Id_PVSystem', 'bus1', 'GPSlat(r)_bus1', 'GPSlon(r)_bus1']]

    BBDD_Buscoords['PVSystem'] = df_PVSystem
    element_list.append('PVSystem')

    workbook_aux = f'BBDD_Buscoords_{prj_name}.xlsx'
    save_BBDD_xlsx(workbook_DSS=workbook_aux, elements_OpenDSS=element_list,
                   BBDD_OpenDSS=BBDD_Buscoords, out_path=save_path)






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openpy_fx_tools_dss',
 'openpy_fx_tools_dss.IMEX_to_DSS',
 'openpy_fx_tools_dss.IMEX_to_DSS.GIS_info_DSS',
 'openpy_fx_tools_dss.IMEX_to_DSS.cim_DSS_cim',
 'openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx',
 'openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs',
 'openpy_fx_tools_dss.IMEX_to_DSS.xml_DSS_xml',
 'openpy_fx_tools_dss.Plots_DSS',
 'openpy_fx_tools_dss.data_from_DSS',
 'openpy_fx_tools_dss.data_from_DSS.parameters',
 'openpy_fx_tools_dss.data_from_DSS.results',
 'openpy_fx_tools_dss.logg_print_alert']

package_data = \
{'': ['*'],
 'openpy_fx_tools_dss': ['Examples/123Bus/DSS_files/*',
                         'Examples/123Bus/xlsx_files/*',
                         'Examples/123Bus/xlsx_files/DSS_files_from_xlsx/*',
                         'Examples/13Bus/DSS_files/*',
                         'Examples/13Bus/xlsx_files/*',
                         'Examples/13Bus/xlsx_files/DSS_files_from_xlsx/*',
                         'Examples/37Bus/DSS_files/*',
                         'Examples/37Bus/xlsx_files/*',
                         'Examples/37Bus/xlsx_files/DSS_files_from_xlsx/*',
                         'Examples/Base_Empty/*'],
 'openpy_fx_tools_dss.data_from_DSS': ['help_DSS/*']}

install_requires = \
['colorama==0.4.6',
 'numpy==1.23.5',
 'opendssdirect-py==0.7.0',
 'openpyxl==3.0.10',
 'pandas==1.5.2',
 'py-dss-interface==1.0.2']

setup_kwargs = {
    'name': 'openpy-fx-tools-dss',
    'version': '0.1.5',
    'description': 'Python functions for extracting, executing routines, etc. from projects modeled in OpenDSS',
    'long_description': '# OpenPy-fx-tools-DSS\nPython functions for extracting, executing routines, etc. from projects modeled in OpenDSS\n# Install\nWith pip\n\n``pip install openpy-fx-tools-dss``\n\n# How to use\nFirst, in the IDE (Integrated Development Environment) of preference, we import the library:\n\n```Python\nimport openpy_fx_tools_dss as fx_dss\n```\n## Examples\nThe library has IEEE example circuits (Table 1), which can be found in the OpenDSS installation files.\nFor the example files the class ``examples_lib()`` is called: \n\n**Table 1.** Sample tests\n\n| **Name**   | **Option** | **DSS files** | **xlsx file** |\n|------------|------------|---------------|---------------|\n| 13BusIEEE  | 1          | x             | x             |\n| 37BusIEEE  | 2          | x             | x             |\n| 123BusIEEE | 3          | x             | x             |\n\n\n```Python\ntest = fx_dss.examples_lib()\n```\n\nTo use the .DSS files:\n\n``DSS_path = test.load_examples_DSS()``\n\nTo use .xlsx file:\n\n``xlsx_path = test.load_examples_xlsx()``\n\n## OpenDSS to xlsx and vice versa \nThe functions that generate OpenDSS scripts to a .xlsx file and vice versa are inside the ``xlsx_DSS_xlsx()`` class and are called in the following way:\n\n``xlsx = fx_dss.xlsx_DSS_xlsx()``\n### Template xlsx\n\nAn .xlsx template with the OpenDSS elements is created by calling the ``create_template_xlsx`` function as follows:\n\n```Python\nxlsx.create_template_xlsx(\n    path_save: str = None, \n    prj_name: str = \'default\', \n    elem_list: list = [], \n    all_elem: bool = True\n)\n```\n### xlsx to OpenDSS\nThe circuit data is entered in the .xlsx template. Each sheet of the template corresponds to the elements that are modeled in OpenDSS. When generating the OpenDSS scripts, only the cells that are not empty are considered.\nFor this purpose, the ``xlsx_to_OpenDSS`` function is called as follows:\n\n```Python\nxlsx.xlsx_to_OpenDSS(\n    xlsx_path=xlsx_path[\'xlsx_path\'],\n    path_save=xlsx_path[\'path_save\'],\n    prj_name=DSS_path[\'prj_name\']\n)\n```\n\n**Important:**\n\n* Do not alter the names of the sheets, and columns, since the element will not be taken into account. \n* The form of data entry must coincide with the OpenDSS specifications.\n\n\n\n\n### OpenDSS to xlsx\nFrom an OpenDSS file, the xlsx template can be generated. For this purpose, the ``OpenDSS_to_xlsx`` function is used as follows:\n\n```Python\nxlsx.OpenDSS_to_xlsx(\n    DSS_path=DSS_path[\'DSS_path\'],\n    path_save=DSS_path[\'path_save\'],\n    prj_name=DSS_path[\'prj_name\']\n)\n```\n\n# License\nLicense: CC BY-NC-SA 4.0\n\n<a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/"><img alt="Creative Commons License" style="border-width:0" src="https://i.creativecommons.org/l/by-nc-sa/4.0/88x31.png" /></a><br />\n\nThis work has a license <a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/">Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License</a>.\n',
    'author': 'Jorge A. Lara S.',
    'author_email': 'jlara@iee.unsj.edu.ar',
    'maintainer': 'Jorge A. Lara S.',
    'maintainer_email': 'jlara@iee.unsj.edu.ar',
    'url': 'https://github.com/jlara6/OpenPy-fx-tools-DSS',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : General_elem_DSS.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_General
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_ORD, _COL_MTY

list_General_DSS = [
    'WireData', 'LineSpacing', 'LineGeometry', 'LineCode', 'XfmrCode', 'CNData', 'GrowthShape', 'LoadShape',
    'PriceShape', 'Spectrum', 'TCC_Curve', 'TSData', 'TShape', 'XYcurve'
]

def General_ORD(DF_elem_DSS: pd.DataFrame, name_class: str):

    if name_class == 'LineCode':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'LoadShape':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'TShape':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'PriceShape':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'XYcurve':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'GrowthShape':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'TCC_Curve':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'Spectrum':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'WireData':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'CNData':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'TSData':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'LineGeometry':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'LineSpacing':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    elif name_class == 'XfmrCode':
        return _COL_ORD(dict_General, DF_elem_DSS, name_class)
    else:
        return DF_elem_DSS

def General_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str):
    if name_class == 'LineCode':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                row_aux = ['units', 'Kron', 'Rg', 'Xg', 'rho', 'neutral', 'LineType']
                value_aux = ['none', 'No', 0.01805, 0.155081, 100, 'nphases', 'value', 'OH']
                for x in zip(row_aux, value_aux):
                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''
                    if x[1] == 'nphases':
                        if DF_elem_DSS[x[0]][index] == DF_elem_DSS[x[1]][index]:
                            DF_elem_DSS[x[0]][index] = ''
        return DF_elem_DSS

    elif name_class == 'LoadShape':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                if DF_elem_DSS['mult'][index] == DF_elem_DSS['Pmult'][index]:
                    DF_elem_DSS['Pmult'][index] = ''
                if DF_elem_DSS['Pmax'][index] == 1:
                    DF_elem_DSS['Pmax'][index] = ''
                if DF_elem_DSS['sinterval'][index] == 3600:
                    DF_elem_DSS['sinterval'][index] = ''
                if DF_elem_DSS['minterval'][index] == 60:
                    DF_elem_DSS['minterval'][index] = ''
        return DF_elem_DSS
    elif name_class == 'TShape':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'PriceShape':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'XYcurve':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'GrowthShape':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'TCC_Curve':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Spectrum':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'WireData':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'CNData':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'TSData':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'LineGeometry':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'LineSpacing':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'XfmrCode':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    else:
        return DF_elem_DSS


def General_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):

    #dss.circuit_set_active_class(name_class)
    #list_property_dss = dss.dsselement_all_property_names()
    list_property = dict_General[name_class]

    if name_class == 'LineCode':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'LoadShape':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'TShape':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'PriceShape':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'XYcurve':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'GrowthShape':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'TCC_Curve':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Spectrum':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'WireData':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'CNData':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'TSData':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'LineGeometry':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'LineSpacing':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'XfmrCode':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return BBDD_elem_DSS, DSS_elem_list





# -*- coding: utf-8 -*-
# @Time    : 26/10/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : logg_alert.py
# @Software: PyCharm

import logging
import os

from colorama import init, Fore, Back, Style

script_path = os.path.dirname(os.path.abspath(__file__))
logger_aux = logging.getLogger(__name__)

orient, indent = "index", 1

add_DSS_empty = True
add_Switch = True

Logg_opt = False
alert_msg = True
print_msg = True

export_csv = True
GIS_Info = True



def update_logg_file(msg: str = None, Typ: int = 0, logger=logger_aux, Logg: bool = Logg_opt, Print: bool = print_msg):
    """
    Creates or updates the log_py file. In addition, it prints the message formatted according to the type.

    :param msg: Message text to be displayed by console or saved in the log_py file
    :param logger: From function logging.getLogger(__name__)
    :param Typ: May be: 0(pass), 1(debug), 2(info) 3(warning), 4(error), and 5 (critical). Default=0
    :param Logg: boolean to enable or disable log_py file. Default=True
    :param Print: Display the message by console. Default=True
    :return: py_open_dsse.log_py
    """
    if Logg:

        log_format = '%(asctime)s - %(name)s - %(levelname)s - %(msg)s'
        # To override the default severity of logging
        logger.setLevel('DEBUG')
        # logger = logging.getLogger(__name__)

        # Use FileHandler() to log_py to a file
        file_handler = logging.FileHandler(f"{script_path}\py_fx_tools_dss.log")
        formatter = logging.Formatter(log_format)
        file_handler.setFormatter(formatter)

        # Don't forget to add the file handler
        logger.addHandler(file_handler)
        if Typ == 1:
            logger.debug(msg)
        if Typ == 2:
            logger.info(msg)
        if Typ == 3:
            logger.warning(msg)
        if Typ == 4:
            logger.error(msg)
        if Typ == 5:
            logger.critical(msg)

    if Print:
        if Typ == 0:
            print(Style.DIM + msg)
        else:
            #print('_' * 64)
            if Typ == 1:
                'debug'
                print(Style.NORMAL + msg + Style.RESET_ALL)
            if Typ == 2:
                'info'
                print(Style.BRIGHT + msg + Style.RESET_ALL)
            if Typ == 3:
                'warning'
                print(Fore.YELLOW + msg + Style.RESET_ALL)
            if Typ == 4:
                'error'
                print(Fore.RED + msg + Style.RESET_ALL)
            if Typ == 5:
                'critical'
                print(Fore.RED + Style.BRIGHT + msg + Style.RESET_ALL)

def alert_messages(msg: str = None, Typ: int = 0, alert: bool = alert_msg):
    """
    Creates or updates the log_py file. In addition, it prints the message formatted according to the type.

    :param msg: Message text to be displayed by console
    :param Typ: May be: 0(pass), 1(debug), 2(info) 3(warning), 4(error), and 5 (critical). Default=0
    :param alert: Display the message by console. Default=True
    :return: py_open_dsse.log_py
    """

    if alert == True:
        if Typ == 0:
            print(Style.DIM + msg)
        else:
            # print('_' * 64)
            if Typ == 1:
                'debug'
                print(Style.NORMAL + msg + Style.RESET_ALL)
            if Typ == 2:
                'info'
                print(Style.BRIGHT + msg + Style.RESET_ALL)
            if Typ == 3:
                'warning'
                print(Fore.YELLOW + msg + Style.RESET_ALL)
            if Typ == 4:
                'error'
                print(Fore.RED + msg + Style.RESET_ALL)
            if Typ == 5:
                'critical'
                print(Fore.RED + Style.BRIGHT + msg + Style.RESET_ALL)